'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

require('./model');

var _inflection = require('inflection');

var _errors = require('./errors');

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var DEFAULT_OPTIONS = {
  convertCase: false
};

/**
 * Entry class for accessing the functionality of Knex-ORM.
 * @property {Object} knex Knex client corresponding to the ORM instance.
 */

var KnexOrm = function () {
  /**
   * Creates a new Knex-ORM instance.
   * @param {Object} knex Knex client instance to which database functions shall
   * be bound.
   * @param {Object} [options] Additional options regarding ORM.
   * @param {boolean} [options.convertCase=false] If set to true, then the ORM
   * will handle letter case conversion for properties automatically (between
   * camelCase and snake_case).
   */

  function KnexOrm(knex, options) {
    var _this = this;

    (0, _classCallCheck3.default)(this, KnexOrm);

    // Initialize private properties
    Object.defineProperty(this, '_models', { value: {} });

    // Store the given Knex client instance and then make it overridable
    this.knex = (0, _assign2.default)({}, knex);

    // Parse and store options
    this.options = (0, _assign2.default)(DEFAULT_OPTIONS, options);
    if (this.options.convertCase) {
      (function () {
        var formatterPrototype = _this.knex.client.Formatter.prototype;

        // Override a Knex query formatter function by extending it
        (function (originalFunction) {
          formatterPrototype._wrapString = function _wrapString(value) {
            return (0, _inflection.underscore)(originalFunction.call(this, value));
          };
        })(formatterPrototype._wrapString);
      })();
    }
  }

  /**
   * Base Model class corresponding to the current ORM instance.
   * @type {Model}
   */


  (0, _createClass3.default)(KnexOrm, [{
    key: 'register',


    /**
     * Registers a static Model object to the list of database objects.
     * @param {Model} model Model to be registered.
     * @param {string} [name] Name under which the Model shall be registered.
     * @throws {DbObjectAlreadyRegisteredError}
     * @returns {Model} The Model which was registered.
     */
    value: function register(model, name) {
      // Determine the Model's name and then check if it's already registered
      var modelName = name || model.name;
      if ((0, _keys2.default)(this._models).indexOf(modelName) >= 0) {
        throw new _errors.DbObjectAlreadyRegisteredError(modelName);
      }

      this._models[modelName] = model;
      return model;
    }
  }, {
    key: 'Model',
    get: function get() {
      if (!this._model) {
        // Clone the original class
        this._model = (0, _utils.requireUncached)('./model');
        this._model._parent = this;
      }

      return this._model;
    }
  }]);
  return KnexOrm;
}();

exports.default = KnexOrm;
//# sourceMappingURL=index.js.map