'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _entries = require('babel-runtime/core-js/object/entries');

var _entries2 = _interopRequireDefault(_entries);

var _set = require('babel-runtime/core-js/set');

var _set2 = _interopRequireDefault(_set);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Represents a query builder which corresponds to a static Model reference.
 * Inherits every query method of the Knex query builder.
 */

var QueryBuilder = function () {
  function QueryBuilder(Model) {
    (0, _classCallCheck3.default)(this, QueryBuilder);

    this.Model = Model;

    Object.defineProperty(this, '_knexQb', {
      writable: true,
      value: Model._parent.knex.from(Model.tableName)
    });

    Object.defineProperty(this, '_relations', { value: new _set2.default() });
  }

  /**
   * Fetches the given related Models of the queryable instance(s).
   * @param {...string} props Relation attributes to be fetched.
   * @returns {QueryBuilder}
   */


  (0, _createClass3.default)(QueryBuilder, [{
    key: 'withRelated',
    value: function withRelated() {
      for (var _len = arguments.length, props = Array(_len), _key = 0; _key < _len; _key++) {
        props[_key] = arguments[_key];
      }

      var relationNames = (0, _utils.flattenArray)(props);
      var relationEntries = (0, _entries2.default)(this.Model.related);

      // Filter the given relations by name if necessary
      if (relationNames.length > 0) {
        relationEntries.filter(function (_ref) {
          var _ref2 = (0, _slicedToArray3.default)(_ref, 1);

          var name = _ref2[0];
          return relationNames.indexOf(name) >= 0;
        });
      }

      // Store the filtered relations
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = (0, _getIterator3.default)(relationEntries), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var _step$value = (0, _slicedToArray3.default)(_step.value, 2);

          var name = _step$value[0];
          var relation = _step$value[1];

          relation.name = name;
          this._relations.add(relation);
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      return this;
    }

    /**
     * Executes the query as a Promise.
     * @param {...Object} args Arguments to be passed to Promise.then().
     * @returns {Promise}
     */

  }, {
    key: 'then',
    value: function then() {
      var _knexQb$then$then,
          _this = this;

      var result = void 0;

      return (_knexQb$then$then = this._knexQb.then(function (res) {
        var awaitableQueries = [];
        result = res;

        // Apply letter case conversion if needed
        if (_this.Model._parent.options.convertCase) {
          result = (0, _utils.camelizeKeys)(result);
        }

        // Convert the result to a specific Model type if necessary
        result = (0, _utils.modelize)(result, _this.Model);

        // Apply each desired relation to the original result
        var _iteratorNormalCompletion2 = true;
        var _didIteratorError2 = false;
        var _iteratorError2 = undefined;

        try {
          for (var _iterator2 = (0, _getIterator3.default)(_this._relations), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            var relation = _step2.value;

            awaitableQueries.push(relation.applyAsync(result));
          }
        } catch (err) {
          _didIteratorError2 = true;
          _iteratorError2 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion2 && _iterator2.return) {
              _iterator2.return();
            }
          } finally {
            if (_didIteratorError2) {
              throw _iteratorError2;
            }
          }
        }

        return _promise2.default.all(awaitableQueries);
      }).then(function () {
        return result;
      })).then.apply(_knexQb$then$then, arguments);
    }

    /**
     * Gets the list of raw queries to be executed, joined by a string separator.
     * @param {string} [separator=\n] Separator string to be used for joining
     * multiple raw query strings.
     * @returns {string}
     */

  }, {
    key: 'toString',
    value: function toString() {
      var separator = arguments.length <= 0 || arguments[0] === undefined ? '\n' : arguments[0];

      // Return a list of query strings to be executed, including Relations
      var result = [this._knexQb.toString()];
      var _iteratorNormalCompletion3 = true;
      var _didIteratorError3 = false;
      var _iteratorError3 = undefined;

      try {
        for (var _iterator3 = (0, _getIterator3.default)(this._relations), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
          var relation = _step3.value;

          // Create the relation query with an empty array of Models
          result.push(relation.createQuery([]).toString());
        }
      } catch (err) {
        _didIteratorError3 = true;
        _iteratorError3 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion3 && _iterator3.return) {
            _iterator3.return();
          }
        } finally {
          if (_didIteratorError3) {
            throw _iteratorError3;
          }
        }
      }

      return result.join(separator);
    }
  }]);
  return QueryBuilder;
}();

// Inherit Knex query methods


exports.default = QueryBuilder;
var _iteratorNormalCompletion4 = true;
var _didIteratorError4 = false;
var _iteratorError4 = undefined;

try {
  var _loop = function _loop() {
    var method = _step4.value;

    QueryBuilder.prototype[method] = function queryMethod() {
      var _knexQb;

      // Update Knex state
      this._knexQb = (_knexQb = this._knexQb)[method].apply(_knexQb, arguments);
      return this;
    };
  };

  for (var _iterator4 = (0, _getIterator3.default)(_config2.default.KNEX_ALLOWED_QUERY_METHODS), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
    _loop();
  }
} catch (err) {
  _didIteratorError4 = true;
  _iteratorError4 = err;
} finally {
  try {
    if (!_iteratorNormalCompletion4 && _iterator4.return) {
      _iterator4.return();
    }
  } finally {
    if (_didIteratorError4) {
      throw _iteratorError4;
    }
  }
}
//# sourceMappingURL=query-builder.js.map