'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _inflection = require('inflection');

var _relationType = require('./enums/relation-type');

var _relationType2 = _interopRequireDefault(_relationType);

var _errors = require('./errors');

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Represents a relation between Models.
 * @property {Model} origin Static Model object which shall be joined with the
 * target.
 * @property {Model} target Static Model object which corresponds to the origin.
 * @property {RelationType} type Type of the relation between 'origin' and
 * 'target'.
 * @property {string} foreignKey The attribute which points to the primary key
 * (ID attribute) of the joinable database table.
 * @private
 */

var Relation = function () {
  function Relation(Origin, Target, type, foreignKey) {
    (0, _classCallCheck3.default)(this, Relation);

    this.origin = Origin;

    // Get the target's registered Model if target is a string
    var modelRegistry = Origin._parent._models;
    this.target = typeof Target === 'string' ? modelRegistry[Target] : Target;

    this.type = type;
    this._foreignKey = foreignKey;
  }

  (0, _createClass3.default)(Relation, [{
    key: 'createQuery',
    value: function createQuery(originInstances) {
      var originAttribute = this.originAttribute;
      var targetAttribute = this.targetAttribute;

      return this.target.query().whereIn(originAttribute, originInstances.length > 0 ? // Pass a mock value if necessary
      originInstances.map(function (model) {
        return model[targetAttribute];
      }) : ['originInstance.' + targetAttribute]);
    }
  }, {
    key: 'applyAsync',
    value: function applyAsync() {
      var _this = this;

      for (var _len = arguments.length, originInstances = Array(_len), _key = 0; _key < _len; _key++) {
        originInstances[_key] = arguments[_key];
      }

      var models = (0, _utils.flattenArray)(originInstances);
      var originAttribute = this.originAttribute;
      var targetAttribute = this.targetAttribute;

      // Create and then execute the query, handling Model bindings
      return this.createQuery(models).then(function (relatedModels) {
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          var _loop = function _loop() {
            var relatedModel = _step.value;

            // Pair up the related Model with its origin
            var foreignValue = relatedModel[originAttribute];
            var origin = models.find(function (model) {
              return model[targetAttribute] === foreignValue;
            });

            if (origin) {
              if (typeof origin[_this.name] === 'undefined') {
                // Initially set the origin's related property
                if (_this.type === _relationType2.default.ONE_TO_MANY) {
                  origin[_this.name] = [relatedModel];
                } else {
                  origin[_this.name] = relatedModel;
                }
              } else {
                // Modify the origin's related property if possible
                if (_this.type === _relationType2.default.ONE_TO_MANY) {
                  origin[_this.name].push(relatedModel);
                } else {
                  throw new _errors.RelationError();
                }
              }
            }
          };

          for (var _iterator = (0, _getIterator3.default)(relatedModels), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            _loop();
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator.return) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }
      });
    }
  }, {
    key: 'foreignKey',
    get: function get() {
      if (!this._foreignKey) {
        // Set the foreign key deterministically
        this._foreignKey = this.type === _relationType2.default.MANY_TO_ONE ? (0, _inflection.underscore)(this.target.name) + '_id' : (0, _inflection.underscore)(this.origin.name) + '_id';

        if (this.origin._parent.options.convertCase) {
          this._foreignKey = (0, _inflection.camelize)(this._foreignKey, true);
        }
      }

      return this._foreignKey;
    }
  }, {
    key: 'originAttribute',
    get: function get() {
      var result = this.type === _relationType2.default.MANY_TO_ONE ? this.target.idAttribute : this.foreignKey;

      return this.origin._parent.options.convertCase ? (0, _inflection.camelize)(result, true) : result;
    }
  }, {
    key: 'targetAttribute',
    get: function get() {
      var result = this.type === _relationType2.default.MANY_TO_ONE ? this.foreignKey : this.origin.idAttribute;

      return this.origin._parent.options.convertCase ? (0, _inflection.camelize)(result, true) : result;
    }
  }]);
  return Relation;
}();

exports.default = Relation;
//# sourceMappingURL=relation.js.map